@@ -6,12 +6,20 @@ set -u
 init_logging() {
     local _abs_libdir="$1"
     local _logfile="$_abs_libdir/$TEMPLATE_REL_MANIFEST_DIR/install.log"
+    rm -f "$_logfile"
+    need_ok "failed to remove old installation log"
+    touch "$_logfile"
+    need_ok "failed to create installation log"
     LOGFILE="$_logfile"
 }
 
 log_line() {
     local _line="$1"
 
+    if [ -n "${LOGFILE-}" -a -e "${LOGFILE-}" ]; then
+    echo "$_line" >> "$LOGFILE"
+    # Ignore errors, which may happen e.g. after the manifest dir is deleted
+    fi
 }
 
 msg() {
@@ -152,7 +160,7 @@ valopt() {
     local doc="$*"
     if [ $HELP -eq 0 ]
     then
-        local uop=$(echo $op | tr 'a-z-' 'A-Z_')
+        local uop=$(echo $op | tr '[a-z]-' '[A-Z]_')
         local v="CFG_${uop}"
         eval $v="$default"
         for arg in $CFG_ARGS
@@ -198,8 +206,8 @@ opt() {
         do
             if [ "$arg" = "--${flag}-${op}" ]
             then
-                op=$(echo $op | tr 'a-z-' 'A-Z_')
-                flag=$(echo $flag | tr 'a-z' 'A-Z')
+                op=$(echo $op | tr '[a-z]-' '[A-Z]_')
+                flag=$(echo $flag | tr '[a-z]' '[A-Z]')
                 local v="CFG_${flag}_${op}"
                 eval $v=1
                 putvar $v
@@ -227,7 +235,7 @@ flag() {
         do
             if [ "$arg" = "--${op}" ]
             then
-                op=$(echo $op | tr 'a-z-' 'A-Z_')
+                op=$(echo $op | tr '[a-z]-' '[A-Z]_')
                 local v="CFG_${op}"
                 eval $v=1
                 putvar $v
@@ -425,8 +433,8 @@ uninstall_components() {
             local _directive
             while read _directive; do
 
-            local _command=${_directive%%:*}
-            local _file=${_directive#*:}
+            local _command=`echo $_directive | cut -f1 -d:`
+            local _file=`echo $_directive | cut -f2 -d:`
 
             # Sanity checks
             if [ ! -n "$_command" ]; then critical_err "malformed installation directive"; fi
@@ -533,8 +541,8 @@ install_components() {
     local _directive
     while read _directive; do
 
-        local _command=${_directive%%:*}
-        local _file=${_directive#*:}
+        local _command=`echo $_directive | cut -f1 -d:`
+        local _file=`echo $_directive | cut -f2 -d:`
 
         # Sanity checks
         if [ ! -n "$_command" ]; then critical_err "malformed installation directive"; fi
@@ -543,24 +551,36 @@ install_components() {
         # Decide the destination of the file
         local _file_install_path="$_dest_prefix/$_file"
 
-        case "$_file" in
-            etc/*)
-                local _f=${_file#"etc/"}
-                _file_install_path="$CFG_DESTDIR_PREFIX/share/doc/cargo/$_f"
-                ;;
-            bin/*)
-                local _f=${_file#"bin/"}
-                _file_install_path="$CFG_BINDIR/$_f"
-                ;;
-            lib/*)
-                local _f=${_file#"lib/"}
-                _file_install_path="$CFG_LIBDIR/$_f"
-                ;;
-            share/man/*)
-                local _f=${_file#"share/man/"}
-                _file_install_path="$CFG_MANDIR/$_f"
-                ;;
-            share/doc/*)
+        if echo "$_file" | grep "^etc/" > /dev/null
+        then
+        local _f="$(echo "$_file" | sed 's/^etc\///')"
+        _file_install_path="$CFG_SYSCONFDIR/$_f"
+        fi
+
+        if echo "$_file" | grep "^bin/" > /dev/null
+        then
+        local _f="$(echo "$_file" | sed 's/^bin\///')"
+        _file_install_path="$CFG_BINDIR/$_f"
+        fi
+
+        if echo "$_file" | grep "^lib/" > /dev/null
+        then
+        local _f="$(echo "$_file" | sed 's/^lib\///')"
+        _file_install_path="$CFG_LIBDIR/$_f"
+        fi
+
+        if echo "$_file" | grep "^share" > /dev/null
+        then
+        local _f="$(echo "$_file" | sed 's/^share\///')"
+        _file_install_path="$CFG_DATADIR/$_f"
+        fi
+
+        if echo "$_file" | grep "^share/man/" > /dev/null
+        then
+        local _f="$(echo "$_file" | sed 's/^share\/man\///')"
+        _file_install_path="$CFG_MANDIR/$_f"
+        fi
+
             # HACK: Try to support overriding --docdir.  Paths with the form
             # "share/doc/$product/" can be redirected to a single --docdir
             # path. If the following detects that --docdir has been specified
@@ -571,17 +591,14 @@ install_components() {
             # contents of those directories are _not_ disjoint. Since this feature
             # is almost entirely to support 'make install' anyway I don't expect
             # this problem to be a big deal in practice.
-                if [ "$CFG_DOCDIR" != "<default>" ]; then
-                    local _f=${_file#"share/doc/"*/}
-                    _file_install_path="$CFG_DOCDIR/$_f"
-                fi
-                ;;
-            share/*)
-                local _f=${_file#"share/"}
-                _file_install_path="$CFG_DATADIR/$_f"
-                ;;
-        esac
-
+            if [ "$CFG_DOCDIR" != "<default>" ]
+            then
+            if echo "$_file" | grep "^share/doc/" > /dev/null
+            then
+            local _f="$(echo "$_file" | sed 's/^share\/doc\/[^/]*\///')"
+            _file_install_path="$CFG_DOCDIR/$_f"
+            fi
+            fi
 
         # Make sure there's a directory for it
         make_dir_recursive "$(dirname "$_file_install_path")"
@@ -598,16 +615,16 @@ install_components() {
 
             verbose_msg "copying file $_file_install_path"
 
-            local mode=644
-            if test -x "$_src_dir/$_component/$_file"; then
-                mode=755
-            fi
-            case "$_file" in
-                bin/*) mode=755
-                       ;;
-            esac
+            maybe_backup_path "$_file_install_path"
+
+            if echo "$_file" | grep "^bin/" > /dev/null || test -x "$_src_dir/$_component/$_file"
+            then
             run cp "$_src_dir/$_component/$_file" "$_file_install_path"
-            run chmod $mode "$_file_install_path"
+            run chmod 755 "$_file_install_path"
+            else
+            run cp "$_src_dir/$_component/$_file" "$_file_install_path"
+            run chmod 644 "$_file_install_path"
+            fi
             critical_need_ok "file creation failed"
 
             # Update the manifest
@@ -620,6 +637,7 @@ install_components() {
 
             verbose_msg "copying directory $_file_install_path"
 
+            maybe_backup_path "$_file_install_path"
 
             run cp -R "$_src_dir/$_component/$_file" "$_file_install_path"
             critical_need_ok "failed to copy directory"
@@ -968,6 +986,7 @@ write_to_file "$TEMPLATE_RUST_INSTALLER_VERSION" \
 critical_need_ok "failed to write installer version"
 
 # Install the uninstaller
+install_uninstaller "$src_dir" "$src_basename" "$abs_libdir"
 
 # Install each component
 install_components "$src_dir" "$abs_libdir" "$dest_prefix" "$components"
